# college_a5.py
# Prof. Lillian Lee (LJL2), Apr 28, 2021


"""
    Defines the College class and the Showdown class.

"""

# Helper function
def stringify_tags(tags):
    """
    Returns: comma-separated string of the tags in list of ints `tags` plus newline
    """
    return ','.join(map(lambda i : str(i),tags)) + "\n"


class College:
    """ Attributes: 
            name [nonempty str, no spaces]: name of this College.
            
            accepted_enrolled: List of unique ints, possibly empty, but not None.
            accepted_not_enrolled: Ditto.
            rejected: Ditto.
            waitlisted: Ditto.

        The four "status lists" have no ints in common.  The ints are presumed to be
        the tags of students. 
    """

    def __init__(self, name):
        """Initializes a new College with name `name` and all four lists of 
        student tags being empty lists."""
        self.name = name
        self.accepted_enrolled = []
        self.accepted_not_enrolled = []
        self.rejected = []
        self.waitlisted = []

    # needed for testing whether student code alters a College
    def __eq__(self, other):
        """Returns True if other is a College with the same (known) attribute
        values, False otherwise."""
        if not isinstance(other, College) or self.name != other.name:
            return False
        return set(self.accepted_enrolled) == set(other.accepted_enrolled) and \
            set(self.accepted_not_enrolled) == set(other.accepted_not_enrolled) and \
            set(self.rejected) == set(other.rejected) and \
            set(self.waitlisted) == set(other.waitlisted)

    def __str__(self):
        """Returns a string that is a pretty-printing of this College"""
        out = self.name + "\n"
        out += "Accepted and enrolled:\n"
        out += "\t" + stringify_tags(self.accepted_enrolled)
        out += "Accepted and not enrolled:\n"
        out += "\t" + stringify_tags(self.accepted_not_enrolled)
        out += "Rejected:\n"
        out += "\t" + stringify_tags(self.rejected)
        out += "Wait-listed:\n"
        out += "\t" + stringify_tags(self.waitlisted)
        return out

    def add_student(self, tag, my_decision, where_enrolled):
        """Adds student to this College's relevant status list, based on my_decision.  

        Preconditions:
            tag: int.
            my_decision: one of "Accepted", "Rejected", "Wait-listed".
            where_enrolled: string naming the College the student enrolled at.
            `tag` is not already in one of this College's status lists.
        """
        right_list = None 
        if my_decision == "Rejected":
            right_list = self.rejected
        elif my_decision == "Wait-listed":
            right_list = self.waitlisted
        elif my_decision == "Accepted" and where_enrolled == self.name:
            right_list = self.accepted_enrolled
        else:
            right_list = self.accepted_not_enrolled
        right_list.append(tag)

    # This method was originally a function in A3, because students wouldn't
    # have known about "self" yet. It was named "was_accepted" then, but
    # that's no longer an appropriate name if called as a College method  
    def accepted(self, tag):
        """Returns True if the student with tag number `tag` was accepted at
        College c; returns False otherwise.

        (For A3, a waitlisted student is not considered accepted even if they
        eventually made it off the waitlist.)

        Preconditions: 
            tag is an int. 
            c is a College object (not None). 
        """
        assert type(tag) == int, \
            "College method accepted(): `tag` should be int, but was " + str(type(tag))

        return tag in self.accepted_enrolled or tag in self.accepted_not_enrolled


# Do not modify this class!
class Showdown:
    """Represents a showdown between two different Colleges.

    Attributes:
        c1: A College object.
        c2: A different College object. (Although we don't check that c1 != c2).

        accepted_at_both: list of tags of students accepted by both c1 and c2.
        enrolled_at_c1: list of tags of students *accepted at both* who enrolled at c1.
        enrolled_at_c2: list of tags of students *accepted at both* who enrolled at c2.
    """
    def __init__(self, c1, c2):
        """Initializes a new Showdown with first and second College attributes
            set to c1 and c2, respectively;  automatically sets 
            accepted_at_both, enrolled_at_c1, and enrolled_at_c2 to new,
            correctly initialized lists.

            Preconditions:
            c1 and c2 are College objects (not None).

        """
        self.c1 = c1
        self.c2 = c2
        self.accepted_at_both = []
        self.enrolled_at_c1 = []
        self.enrolled_at_c2 = []

        # We'll check all students accepted by c1. There are two cases.
        # Case 1: c1_student didn't enroll at c1
        for c1_student in c1.accepted_not_enrolled:
            if c2.accepted(c1_student):
                self.accepted_at_both.append(c1_student)
                if c1_student in c2.accepted_enrolled:
                    self.enrolled_at_c2.append(c1_student)
                # o.w., c1_student accepted to c2 but didn't enroll
            # o.w., c1_student wasn't even accepted at c2, doesn't get added anywhere

        # Case 2: student did enroll at c1. 
        for c1_student in c1.accepted_enrolled:
            if c2.accepted(c1_student):
                self.accepted_at_both.append(c1_student)
                self.enrolled_at_c1.append(c1_student)

    def print_stats(self):
        """Print out stats for this Showdown.
        """
        n_both = len(self.accepted_at_both)
        n_c1 = len(self.enrolled_at_c1)
        n_c2 = len(self.enrolled_at_c2)
        n_chose_1_or_2 = n_c1 + n_c2

        out = self.c1.name + " vs. " + self.c2.name + "\n"
        out += "Number of students accepted at both: " + str(n_both) + "\n"
        out += "Number of such students who enrolled at one of them: " + \
            str(n_chose_1_or_2) + "\n"
        if n_chose_1_or_2 > 0:
            out += "% who chose " + self.c1.name + " over " + self.c2.name + ": "
            out += str(round(100.0*float(n_c1)/n_chose_1_or_2,2)) + "\n"
            out += "% who chose " + self.c2.name + " over " + self.c1.name + ": "
            out += str(round(100.0*n_c2/n_chose_1_or_2,2))+ "\n"
        print(out)

    def __str__(self):
        """Returns: string that is a pretty-printing of this Showdown."""
        out = self.c1.name + " vs. " + self.c2.name + "\n"

        slist_map = {"accepted at both": self.accepted_at_both,
                    "accepted at both, enrolled at "+self.c1.name: self.enrolled_at_c1,
                    "accepted at both, enrolled at "+self.c2.name: self.enrolled_at_c2}

        for slistname in slist_map:
            out += slistname + ": " + stringify_tags(slist_map[slistname]) 

        return out

##### end of definition of class Showdown



